@@ -1,3 +1,4 @@
+from concurrent.futures import ThreadPoolExecutor
 from functools import wraps
 from re import compile
 from threading import RLock
@@ -47,6 +48,7 @@ class CPDLC:
         _network (Optional[Network]): Hoppie ACARS network
         _client (httpx.Client): httpx client
         _state_lock (threading.RLock): global lock
+        _thread_pool (concurrent.futures.ThreadPoolExecutor) : thread pool
 
     Examples:
         # Create CPDLC client instance\n
@@ -95,9 +97,11 @@ class CPDLC:
         cpdlc.cpdlc_logout()\n
     """
 
-    def __init__(self):
+    def __init__(self, max_workers: int = 8):
         """
         Constructor for CPDLC class
+        Args:
+            max_workers (int): Maximum number of threads
         """
         logger.trace("CPDLC client initializing")
         self._service_initialization = False
@@ -116,11 +120,14 @@ class CPDLC:
         self._cpdlc_atc_info_update_callback: Optional[Callable[[], None]] = None
         self._cpdlc_disconnect_callback: Optional[Callable[[], None]] = None
         self._network: Network = Network.UNKNOWN
-        self._client = Client(timeout=10.0)
         self._state_lock = RLock()
+        self._client: Optional[Client] = None
+        self._thread_pool = ThreadPoolExecutor(max_workers=max_workers)
+        logger.trace("CPDLC client initialized")
+        self._thread_pool.submit(lambda: self.client)
 
     def __del__(self):
-        if hasattr(self, '_client'):
+        if hasattr(self, '_client') and self._client:
             self._client.close()
 
     # Getter and Setter
@@ -204,6 +211,15 @@ class CPDLC:
         self._poller.set_interval(min_interval, max_interval)
 
     # Properties
+    @property
+    def client(self) -> Client:
+        if self._client is None:
+            logger.trace("httpx client initializing")
+            with self._state_lock:
+                self._client = Client(timeout=10)
+            logger.trace("Client initialized")
+        return self._client
+
     @property
     def callsign(self) -> str:
         return self._callsign
@@ -277,17 +293,28 @@ class CPDLC:
             raise ParameterError("Callsign is required")
         if self._login_code is None:
             raise ParameterError("Login code is required")
-        if not self._ping_station():
-            logger.error(f"CPDLC init failed. Connection error")
-            raise InitializationError()
-        logger.debug(f"CPDLC init complete. Connection OK")
-        if self._email is None:
-            logger.trace(f"Half service provide due to missing email")
-            self._service_level = ServiceLevel.HALF
-        else:
-            logger.trace(f"Full service provided")
-            self._service_level = ServiceLevel.FULL
-            self._network = self.get_network()
+
+        def check_connection():
+            if not self._ping_station():
+                logger.error(f"CPDLC init failed. Connection error")
+                raise InitializationError()
+            logger.debug(f"CPDLC init complete. Connection OK")
+
+        def check_service_level():
+            if self._email is None:
+                logger.trace(f"Half service provide due to missing email")
+                self._service_level = ServiceLevel.HALF
+            else:
+                logger.trace(f"Full service provided")
+                self._service_level = ServiceLevel.FULL
+                self._network = self.get_network()
+
+        result_1 = self._thread_pool.submit(check_connection)
+        result_2 = self._thread_pool.submit(check_service_level)
+
+        result_1.result()
+        result_2.result()
+
         self.start_poller()
         self._service_initialization = True
 
@@ -337,14 +364,18 @@ class CPDLC:
         Args:
             message (str): message
         """
-        logger.trace(f"Message received : {message}")
-        for callback in self._message_receiver_callbacks:
+
+        def execute(fn: Callable[[AcarsMessage], None]) -> None:
             try:
                 logger.trace(f"Callback executed: {callback.__name__}")
-                callback(message)
+                fn(message)
             except Exception as e:
                 logger.error(f"Exception occurred while calling callback: {e}")
 
+        logger.trace(f"Message received : {message}")
+        for callback in self._message_receiver_callbacks:
+            self._thread_pool.submit(execute, callback)
+
     def listen_message_sender(self):
         """
         Add callback to send message
@@ -370,14 +401,18 @@ class CPDLC:
             to (str): message send to
             message (str): message
         """
-        logger.trace(f"Message send to {to}: {message}")
-        for callback in self._message_sender_callbacks:
+
+        def execute(fn: Callable[[str, str], None]) -> None:
             try:
                 logger.trace(f"Callback executed: {callback.__name__}")
-                callback(to, message)
+                fn(to, message)
             except Exception as e:
                 logger.error(f"Exception occurred while calling callback: {e}")
 
+        logger.trace(f"Message send to {to}: {message}")
+        for callback in self._message_sender_callbacks:
+            self._thread_pool.submit(execute, callback)
+
     # Decorators
 
     @staticmethod
@@ -481,7 +516,7 @@ class CPDLC:
             NetworkError: Communication failure
         """
         try:
-            return self._client.post(url, data=data)
+            return self.client.post(url, data=data)
         except RequestError as e:
             logger.error(f"Network request failed: {e}")
             raise NetworkError("Network communication failed") from e
